package com.incubator.game.handler;

import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 登录游戏服
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.LoginReq_VALUE)
public class LoginGameHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.LoginRes_VALUE);

		// 判断参数
		String token = request.getDataValue("token", String.class, "");
//		if (token.isEmpty()) {
//			logger.info("登录token为空");
//			response.setCode(ProtoCommon.Code.PARAMETER_ERR_VALUE);
//			return;
//		}
		// 根据token获取用户信息

		String username = request.getDataValue("username", String.class, "");

		// 创建player
		String player_id = UUID.randomUUID().toString();
		PlayerInfoPO playerInfoPO = new PlayerInfoPO();
		playerInfoPO.playerId = player_id;
		playerInfoPO.name = "玩家" + username;
		playerInfoPO.sex = 0;
		playerInfoPO.exp = 3000;
		playerInfoPO.vipLevel = 1;
		playerInfoPO.masterScore = 9000;
		playerInfoPO.masterScoreLevel = 2;
		playerInfoPO.gold = 10000;
		playerInfoPO.diamond = 20000;
		playerInfoPO.startTimeVip = "";
		playerInfoPO.endTimeVip = "";
		playerInfoPO.gps = "";
		playerInfoPO.gpsAddr = "";
		playerInfoPO.foca = 20;
		playerInfoPO.ranking = 20;
		playerInfoPO.figureId = 1;
		playerInfoPO.realName = "李晓强";
		playerInfoPO.phoneNum = "13800000001";
		playerInfoPO.idCard = "42100119000101000X";
		playerInfoPO.friendMasterNum = 1000;
		playerInfoPO.friendMasterRate = 60;
		playerInfoPO.tradPoints = 2000;
		playerInfoPO.tradPointsRate = 30;
		playerInfoPO.tradMatchNum = 3000;
		playerInfoPO.tradMatchRate = 20;
		playerInfoPO.noReshNum = 10000;
		playerInfoPO.noReshRate = 70;
//		playerInfoPO.token = token;
		Player player = PlayerUtil.createPlayer(playerInfoPO);

		// 绑定session
		player.bind(session, 0L, player.getId());
		logger.info("玩家进入游戏 playerId={}", player.getId());

		// 登录数据
		player.onLogin();

		// 正常返回
		Map<String, Object> map = new HashMap<>();
		map.put("playerInfo", ProtoUtil.playerInfoToMessage(player));
		// 玩家绑定的房间号
		map.put("roomId", player.data.roomId);
		// 玩家绑定的官方比赛id
		map.put("contestId", player.data.contestId);
		// 官方比赛信息列表
		map.put("contestInfo", ProtoUtil.contestListToMessage());
		response.setData(map);
	}
}
