package com.incubator.game.util;

import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.player.Player;
import com.incubator.game.room.JDGDRoom;
import com.incubator.game.room.Room;

import java.util.*;

/**
 * 消息工具类
 */
public final class ProtoUtil {

    /**
     * 玩家信息转消息
     *
     * @param player
     * @return
     */
    public static Map<String, Object> playerInfoToMessage(Player player) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("exp", player.data.exp);
        data.put("vipLevel", player.data.vipLevel);
        data.put("masterScore", player.data.masterScore);
        data.put("masterScoreLevel", player.data.masterScoreLevel);
        data.put("gold", player.data.gold);
        data.put("diamond", player.data.diamond);
        data.put("startTimeVip", player.data.startTimeVip);
        data.put("endTimeVip", player.data.endTimeVip);
        data.put("gps", player.data.gps);
        data.put("gpsAddr", player.data.gpsAddr);
        data.put("foca", player.data.foca);
        data.put("ranking", player.data.ranking);
        data.put("figureId", player.data.figureId);
        data.put("realName", player.data.realName);
        data.put("phoneNum", player.data.phoneNum);
        data.put("idCard", player.data.idCard);
        data.put("friendMasterNum", player.data.friendMasterNum);
        data.put("friendMasterRate", player.data.friendMasterRate);
        data.put("tradPoints", player.data.tradPoints);
        data.put("tradPointsRate", player.data.tradPointsRate);
        data.put("tradMatchNum", player.data.tradMatchNum);
        data.put("tradMatchRate", player.data.tradMatchRate);
        data.put("noReshNum", player.data.noReshNum);
        data.put("noReshRate", player.data.noReshRate);
        return data;
    }

    /**
     * 房间信息转消息
     *
     * @param room
     * @param player 本人
     * @param opPlayer 当前操作玩家
     * @return
     */
    public static Map<String, Object> roomToMessage(Room room, Player player, Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", room.roomId);
        data.put("type", room.type);
        data.put("ownerId", room.ownerId);
        data.put("state", room.state);
        data.put("time", room.time);
        // todo 后面这里写法要调整
        if (room.type == 1) {
            data.put("maxRound", ((JDGDRoom) room).data.maxRound);
            data.put("curRound", ((JDGDRoom) room).data.curRound);
            data.put("zhuangPos", ((JDGDRoom) room).data.zhuangPos);
            // 当前级牌点数
            data.put("curLevelPoint", ((JDGDRoom) room).data.curLevelPoint);
            data.put("opId", opPlayer != null ? opPlayer.getId() : "");
            data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
            data.put("curDiscardList", ((JDGDRoom) room).data.curDisCardList);
            data.put("curDisCardType", ((JDGDRoom) room).data.curDisCardType);
            data.put("curDiscardPlayer", ((JDGDRoom) room).data.disCardPlayer != null ? ((JDGDRoom) room).data.disCardPlayer.getId() : "");
            data.put("curDiscardSex", ((JDGDRoom) room).data.disCardPlayer != null ? ((JDGDRoom) room).data.disCardPlayer.data.sex : 0);
            data.put("myInfo", playerRoomToMessage(room, player, true));
            List<Map<String, Object>> otherInfo = new ArrayList<>();
            for (Player tmPlayer : ((JDGDRoom) room).data.playerMap.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    otherInfo.add(playerRoomToMessage(room, tmPlayer, false));
                }
            }
            data.put("otherInfo", otherInfo);
        }
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param room
     * @param player 玩家
     * @param flag 是否本人
     * @return
     */
    public static Map<String, Object> playerRoomToMessage(Room room, Player player, boolean flag) {
        // todo 后面这里写法要调整
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 级数显示 [我方级数, 对方级数]
        data.put("levelPointView", player.data.levelPointView);
        // 队友Id
        data.put("teammateId", player.data.teammateId);
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", flag? player.data.remainCards : "");
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        data.put("cardInfo", cardInfo);
        return data;
    }

    /**
     * 规则信息转消息
     *
     * @param gameRule 游戏规则
     * @return
     */
    public static Map<String, Object> gameRuleToMessage(GMStruct.GameRule gameRule) {
        Map<String, Object> data = new HashMap<>();
        // 掼蛋
        data.put("gameType", 1);
        // 玩法1 1:连续打，2:按局数，3:按时间
        data.put("playingTypeOne", new int[]{1,2,3});
        // 玩法2 1:A必打 2:J必打 3:8必打 4:5必打
        data.put("playingTypeTwo", new int[]{1,2,3,4});
        // 升级规则 1:123级，2:124级，3:135级，4:246级
        data.put("upgradeType", new int[]{1,2,3,4});
        // 翻山规则 1:有头游无末游过A结束，2:有头游过A结束，3:3局不过A判负结束，4:3局不过A从2开始
        data.put("flippingType", new int[]{1,2,3,4});
        // 进贡规则 1:进贡，2:不进贡（头游先出）
        data.put("advanceType", new int[]{1,2});
        // 超时规则 1:过牌 2:待机 3:托管
        data.put("timeoutType", new int[]{1,2,3});
        // 语音规则 1允许 2禁用
        data.put("voiceType", new int[]{1,2});
        // 记牌器 1允许 2禁用
        data.put("recordType", new int[]{1,2});
        // 观看规则 1第一视角 2上帝视角 3上帝&第一视角 4禁用
        data.put("watchType", new int[]{1,2,3,4});
        // 人数模式 1:2人 2:4人
        data.put("PlayerNum", new int[]{1,2});
        return data;
    }

    /**
     * 官方比赛集合转消息
     *
     * @return
     */
    public static List<Object> contestListToMessage() {
        List<Object> list = new ArrayList<>();
        for (Contest contest : ContestService.getInstance().contestMap.values()) {
            if (contest != null) {
                list.add(ProtoUtil.contestInfoToMessage(contest));
            }
        }
        return list;
    }

    /**
     * 官方比赛集合转消息
     *
     * @return
     */
    public static Map<String, Object> contestInfoToMessage(Contest contest) {
        Map<String, Object> data = new HashMap<>();
        if (contest != null) {
            data.put("contestId", contest.data.contestId);
            data.put("contestName", contest.data.name);
            data.put("contestType", contest.data.contestType);
            data.put("contestPlayType", contest.data.type);
            data.put("contestLevel", contest.data.level);
            data.put("contestTime", contest.data.startTime);
            data.put("contestState", contest.data.state);
            data.put("contestPhotoUrl", contest.data.url);
            data.put("regisCount", 0);
            data.put("regisMoney", "");
            data.put("regisMoneyType", "");
            List<Map<String, Object>> regisList = new ArrayList<>();
            if (!contest.actors.isEmpty()) {
                Map<String, Object> playerInfo;
                for (Player player : contest.actors.values()) {
                    if (player == null) {
                        continue;
                    }
                    playerInfo = new HashMap<>();
                    playerInfo.put("id", player.getId());
                    playerInfo.put("name", player.data.name);
                    playerInfo.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
                    playerInfo.put("sex", player.data.sex);
                    regisList.add(playerInfo);
                }
            }
            data.put("regisList", regisList);
        }
        return data;
    }
}
